//+------------------------------------------------------------------+
//| DLobotomy.mq4.mq4
//| Copyright  2007, Alexander Enchevich, Stephen "the Neuro Surgeon" Davis
//| http://www.metaquotes.net
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Alexander Enchevich, Stephen \"the Neuro Surgeon\" Davis"
#property link      "http://www.metaquotes.net"

#include <CommonLib.mqh>
#import "CommonLib.ex4"

#include <SeriesAnalysisLib.mqh>
#import "SeriesAnalysisLib.ex4"

#include <TradingLib.mqh>
#import "TradingLib.ex4"

#define _MagicNumber 200702031409
#define LOOKBACK	3
#define LOOKBACKTREND	4

#define SLIPPAGE_ON_ENTER	3
#define SLIPPAGE_ON_CLOSE	4

#define TRADE_PERIOD	PERIOD_H4
#define TREND_PERIOD	PERIOD_D1


// counters for SelectFirst... SelectNext... functions
static int s_iCurrBuyPosition=0;
static int s_iCurrSellPosition=0;

// Maximum % of account that we are willing to risk when entering a 
// trade, e.g. 0.05 means 5%
extern double UseableMargin	= 0.05;

// Are we looking at the forming candle (0) or the previous, completed candle (1)
// when calculating RSI break-trhoughs and MA trends
extern int g_iGoBack=1;

extern string MAMethods="0:Simple, 1:Exponential, 2:Smooth, 3:Linear Weighted";
extern string PriceCandleOptions="0:CLOSE, 1:OPEN, 2:HIGH, 3:LOW, 4:MEDIAN (H+L)/2, 5:TYPICAL (H+L+C)/3; 6:WEIGHTED";

// Fast MA is needed when using either of 1MA and 2MA methods for Trend Detection
extern int g_iFastMAMethod=MODE_SMMA;
extern int g_iFastMAPriceMethod=PRICE_MEDIAN;
extern int g_iFastMAPeriod=6;

// Slow MA only needed when using the 2MA Trend Detection method
extern int g_iSlowMAMethod=MODE_EMA;
extern int g_iSlowMAPriceMethod=PRICE_OPEN;
extern int g_iSlowMAPeriod=4;

// RSI is used always - when it crosses the corresponding 4 levels 
// (the 4 doubles, see below), we enter/exit the corresponding trade
extern int g_iRSIPriceMethod=PRICE_TYPICAL;
extern int g_iRSIPeriod=5;

extern double g_fRSILevelBuy=40.0;
extern double g_fRSILevelExitBuy=80.0;
extern double g_fRSILevelExitStrongBuy=90.0;
extern double g_fRSILevelSell=55.0;
extern double g_fRSILevelExitSell=14.0;
extern double g_fRSILevelExitStrongSell=8.0;

//+------------------------------------------------------------------+
double arrRSILevelBuy[LOOKBACK];
double arrRSILevelExitBuy[LOOKBACK];
double arrRSILevelExitStrongBuy[LOOKBACK];
double arrRSILevelSell[LOOKBACK];
double arrRSILevelExitSell[LOOKBACK];
double arrRSILevelExitStrongSell[LOOKBACK];

double arrRSI	[LOOKBACK]; // holds RSI calculations for LOOKBACK ticks back
double arrTrend	[LOOKBACKTREND];
//+------------------------------------------------------------------+
string	g_strChartSymbol = "";
int 	g_iChartPeriod = -1;
bool 	g_bDisabled = false;

//+------------------------------------------------------------------+
// Calculates the current trend. iGoBack can be 0 or 1 and its meaning is 
// whether to use the current, forming candle when calculating the trend
// (iGoBack=0) or to only work with closed candles (iGoBack=1)
double TrendDirection(int iGoBack)
{
//	return (TrendDirection_2MAs(iGoBack));
	return (TrendDirection_1MA(iGoBack));
}
//+------------------------------------------------------------------+
int OpenAtMarket (  int iOrderType,
					double fLots,
					double fPrice,
					int iSlippage, 
					int iMagicNumber )
{
	if(iOrderType != OP_BUY && iOrderType != OP_SELL)
		return(-1);

// TODO : check for other limitations of OrderSend
// TODO : Implement context locking!
	if(fLots < 0.1)
		return(-1);

	string strOperation = OperationToStr( iOrderType );

	double fStopLoss = Ask-10000*Point; // no stop loss!
	double fTakeProfit = Ask+10000*Point; // no take profit!
	color iColor = RoyalBlue;
	
	if( OP_SELL == iOrderType )
	{		
		fStopLoss = Bid+10000*Point;
		fTakeProfit = Bid-10000*Point;
		iColor = OrangeRed;
	}

	int iTicket = 
	OrderSend(	Symbol(), iOrderType, 
				NormalizeDouble(fLots, 1), NormalizeDouble(fPrice, 4),
				iSlippage, fStopLoss, fTakeProfit,
				strOperation, iMagicNumber, 0, iColor );

	if( -1 == iTicket )
	{
		Alert("OpenAtMarket: OrderSend Error ", GetLastError() );
	}
	
	return(iTicket);
}
//+------------------------------------------------------------------+
void CloseAllPositions( int iOrderType, 
						int iMagicNumber,
						double fPreferredClosePrice,
						int iSlippage, 
						color iColor )
{
	for( int iOrderPos=OrdersTotal(); iOrderPos>0; iOrderPos-- )
	{
		if( !OrderSelect( iOrderPos-1, SELECT_BY_POS ) )
		{
			Alert ( "CloseAllPositions: Unable to select order at position ", 
					iOrderPos-1, " Error # ", GetLastError() );
		}
		else if(OrderSymbol() == Symbol() &&
				OrderMagicNumber() == iMagicNumber )
		{
			bool bCloseAll = (iOrderType==-1);

			if( bCloseAll || OrderType() == iOrderType )
			{
				int iTicket = OrderTicket();
				if( !OrderClose(iTicket, OrderLots(), fPreferredClosePrice, iSlippage, iColor) )
				{
					Alert ( "CloseAllPositions: Unable to Close Order #", iTicket,
							" at position ", iOrderPos-1,
							" Error # ", GetLastError() );
				}
			}
		}
	}
}
//+------------------------------------------------------------------+
void DrawTrendArrow(double iTrend)
{
	string strObjName = "Trend_"+
						TimeToStr(TimeCurrent(), TIME_DATE)+"_"+
						DoubleToStr( TimeHour(TimeCurrent()), 0 );

	if(ObjectFind(strObjName) == -1 )
	{
		ObjectCreate(strObjName, OBJ_ARROW, 0, TimeCurrent(), Ask+0.005, 0, 0, 0, 0);
		ObjectSet( strObjName, OBJPROP_ARROWCODE, SYMBOL_STOPSIGN );
		ObjectSet( strObjName, OBJPROP_COLOR, Yellow);
	}

	if(iTrend>0.0)
	{
		ObjectSet( strObjName, OBJPROP_ARROWCODE, SYMBOL_ARROWUP );
		ObjectSet( strObjName, OBJPROP_COLOR, Blue );
	}
	else if(iTrend<0.0)
	{
		ObjectSet( strObjName, OBJPROP_ARROWCODE, SYMBOL_ARROWDOWN );
		ObjectSet( strObjName, OBJPROP_COLOR, Red );
	}
}
//+------------------------------------------------------------------+
// Trend is UP when last close is greater than the chosen MA
// Trend is DOWN when last close is less than the chosen MA
// MA period is usualy chosen between 3 and 7
double TrendDirection_1MA(int iGoBack)
{
	double fMA=iMA( g_strChartSymbol, TRADE_PERIOD,
					g_iFastMAPeriod, 0,
					g_iFastMAMethod, g_iFastMAPriceMethod,
					iGoBack);

	return(Close[iGoBack]-fMA);
}
//+------------------------------------------------------------------+
// Trend UP when Fast MA > Slow MA and vice versa 
// MA period is usualy 3 to 7
double TrendDirection_2MAs(int iGoBack)
{
	double fSlowMA=iMA( g_strChartSymbol, TREND_PERIOD,
						g_iSlowMAPeriod, 0,
						g_iSlowMAMethod, g_iSlowMAPriceMethod,
						iGoBack );

	double fFastMA=iMA( g_strChartSymbol, TREND_PERIOD,
						g_iFastMAPeriod, 0,
						g_iFastMAMethod, g_iFastMAPriceMethod,
						iGoBack );

	return(fFastMA-fSlowMA);
}
//+------------------------------------------------------------------+
bool IsTrendUp(double fTrend)
{
	bool bUp = false;

	// values close to zero not considered indicative or trend even if >0
	if( !IsZero(fTrend) )
		bUp = (fTrend > 0);

	return(bUp);
}
//+------------------------------------------------------------------+
bool IsTrendDown(double fTrend)
{
	bool bDown = false;

	// values close to zero not considered indicative or trend even if <0
	if( !IsZero(fTrend) )
		bDown = (fTrend < 0);

	return(bDown);
}
//+------------------------------------------------------------------+
// On every new candle we calculate the last candle trend and put it into arrTrend.
double CalcTrend()
{
	static double fCurrTrend = 0.0;
	
	// bRecalc tells us whether to recalculate on every tick (g_iGoBack=0)
	//	or only at beginning of new candle, by examining the previous
	//	closed candle (g_iGoBack=1). The values for g_iGoBack - 0 amd 1
	//	reflect how far back to go in the historic price arrays, like
	//	Close[] for example. Close[0] means current close, whereas 
	//	Close[1] means close of previous candle.
	bool bRecalc = true; 

	// GoBack governs whether to consider curr. candle when determining trend
	if( g_iGoBack != 0 ) 
		bRecalc = IsCandleStart(1);

	if(bRecalc)
	{
		for( int i=LOOKBACKTREND-1; i>0; i-- )
		{
			arrTrend[i] = arrTrend[i-1];
		}

		arrTrend[0] = TrendDirection(g_iGoBack);
		DrawTrendArrow( arrTrend[0] );
	
		if( Sign(arrTrend[0]) != Sign(fCurrTrend ) )
		{
			if(	Sign(arrTrend[0]) == Sign(arrTrend[1]) )
			{
				fCurrTrend = arrTrend[0];
			}
		}
	}

	return(fCurrTrend);	
}
//+------------------------------------------------------------------+
// On every tick we calculate the curr RSI and put it into arrRSI.
// arrRSI is ordered like a series - at index 0 is the latest RSI value,
// every time we add a new value we shift the rest backwards (losing the last)
void CalcRSI()
{
	// bRecalc tells us whether to recalculate on every tick (g_iGoBack=0)
	// or only at beginning of new candle, by examining the previous
	// closed candle (g_iGoBack=1). The values for g_iGoBack - 0 amd 1
	// reflect how far back to go in the historic price arrays, like
	// Close[] for example. Close[0] means current close, whereas 
	// Close[1] means close of previous candle.
	bool bRecalc = true; 

	// GoBack governs whether to consider curr. candle when determining trend
	if( g_iGoBack != 0 ) 
		bRecalc = IsCandleStart(1);

	if(bRecalc)
	{
		for( int i=LOOKBACK-1; i>0; i--)
			arrRSI[i] = arrRSI[i-1];

		arrRSI[0]=iRSI( g_strChartSymbol, g_iChartPeriod,
						g_iRSIPeriod,
						g_iRSIPriceMethod,
						g_iGoBack );
	}
}
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
	Print("BeginInit");
	ObjectCreate("Begin", OBJ_VLINE, 0, TimeCurrent(), 0, 0, 0, 0, 0);
	ObjectSet( "Begin", OBJPROP_COLOR, Lime );
	//----
	g_strChartSymbol=Symbol();
	if( g_strChartSymbol != "EURUSD" && g_strChartSymbol != "GBPUSD" )
	{
		g_bDisabled = true;
	}

	g_iChartPeriod=Period();
	if( !g_bDisabled ) // if chart Symbol was OK check Period
		if( g_iChartPeriod != TRADE_PERIOD )
		{
			g_bDisabled = true;
		}

	if( g_bDisabled )
	{
		string strWhyDisabled = "Please only use this advisor with a EURUSD or GBPUSD ";
		strWhyDisabled = StringConcatenate( strWhyDisabled, NormalizeDouble(TRADE_PERIOD/60, 0), "-hr chart!" );
		
		MessageBox( strWhyDisabled,
					"Advisor Disabled!");
	}
	else
	{
		ArrayInitialize(arrRSILevelBuy,		g_fRSILevelBuy);
		ArrayInitialize(arrRSILevelExitBuy, g_fRSILevelExitBuy);
		ArrayInitialize(arrRSILevelSell,	g_fRSILevelSell);
		ArrayInitialize(arrRSILevelExitSell,g_fRSILevelExitSell);
		ArrayInitialize(arrTrend, 0.0);
		
		CalcRSI();
		CalcTrend();
		
		Print("--------------------");
		Print(" FastMAMethod ", g_iFastMAMethod);
		Print(" SlowMAMethod ", g_iSlowMAMethod);
		Print(" FastMAPriceMethod ", g_iFastMAPriceMethod);
		Print(" SlowMAPriceMethod ", g_iSlowMAPriceMethod);
		Print(" FastMAPeriod ", g_iFastMAPeriod);
		Print(" SlowMAPeriod ", g_iSlowMAPeriod);

		Print(" RSIPriceMethod ", g_iRSIPriceMethod);
		Print(" RSIPeriod ", g_iRSIPeriod);
		Print(" RSILevelBuy ", g_fRSILevelBuy);
		Print(" RSILevelExitBuy ", g_fRSILevelExitBuy);
		Print(" RSILevelSell ", g_fRSILevelSell);
		Print(" RSILevelExitSell ", g_fRSILevelExitSell);
		Print(" LookBack N candles?", g_iGoBack);
		Print("--------------------");
	}

	Print("End Init");
	return(0);
}
//+------------------------------------------------------------------+
bool HaveBuySignal()
{
	bool bBuy = IsCrossUp(0, LOOKBACK-1, arrRSI, arrRSILevelBuy);
	
	if( bBuy )
	{
		Print ( "BUY Signal! Buy Level is ", g_fRSILevelBuy,
				"; arrRSI = ", ArrDToStr(arrRSI, 2) );
	}
	return( bBuy );
}
//+------------------------------------------------------------------+
bool HaveSellSignal()
{
	bool bSell = IsCrossDown(0, LOOKBACK-1, arrRSI, arrRSILevelSell);
	if( bSell )
	{
		Print ( "SELL Signal! Sell Level is ", g_fRSILevelSell,
				"; arrRSI = ", ArrDToStr(arrRSI, 2) );
	}
	return( bSell );
}
//+------------------------------------------------------------------+
bool HaveTakeProfitSignal(int iOrderType)
{
	if( OP_BUY == iOrderType )
	{
		if( IsCrossDown(0, LOOKBACK-1, arrRSI, arrRSILevelExitBuy) )
			return(true);

		if( IsCrossUp(0, LOOKBACK-1, arrRSI, arrRSILevelExitStrongBuy) )
			return(true);
	}
	else
	if( OP_SELL == iOrderType )
	{
		if( IsCrossUp(0, LOOKBACK-1, arrRSI, arrRSILevelExitSell) )
			return(true);

		if( IsCrossDown(0, LOOKBACK-1, arrRSI, arrRSILevelExitStrongSell) )
			return(true);
	}

	return (false);
}
//+------------------------------------------------------------------+
void InspectBUYPositions(double fTrend)
{
	bool bHaveBuyOrders = SelectFirstBuyPosition( _MagicNumber );

	if( !bHaveBuyOrders ) return;

	color colorCloseBuy = Black; // Black means "don't close".
	if( IsTrendDown(fTrend))
	{
		Print("Closing All Buy position(s) - trend changed!");
		colorCloseBuy = Red;
	}
	// Min(LOOKBACK-1, 1) wil guarantee proper operation in both real-Time
	// and closed-candle trading modes
	else if( IsCrossDown(0, MathMin(LOOKBACK-1, 1), arrRSI, arrRSILevelExitSell) )
	{
		Print("Closing All Buy position(s) to cut losses");
		colorCloseBuy = Orange;
	}

	// are we closing all BUY positions because things went bad?
	if( colorCloseBuy != Black )
	{
		CloseAllPositions ( OP_BUY,
							_MagicNumber, 
							Bid,
							SLIPPAGE_ON_CLOSE, 
							colorCloseBuy );
	}
	else // are we closing positions for profit?
	{
		bool bHaveTakeProfit = HaveTakeProfitSignal( OP_BUY );
		colorCloseBuy = Green;

		if( bHaveTakeProfit )
		{
			while(bHaveBuyOrders)
			{
				OrderSelect(OrdersTotal()-1, SELECT_BY_POS);
				int iTicket = OrderTicket();
				Print("Closing Buy position ", iTicket, " of ", OrdersTotal(), " for profit");
				double fLotsToCloseThisOrder = LotSizeNormalized(OrderLots()/2);
				
				if( !OrderClose(iTicket, OrderLots(), 
								Bid, SLIPPAGE_ON_CLOSE, colorCloseBuy) )
				{
					Alert ( "InspectBUYPositions: Unable to Close BUY Order #",
							iTicket, ". Error # ", GetLastError() );
				}

				bHaveBuyOrders = SelectNextBuyPosition( _MagicNumber );
			}
		}
	}
}
//+------------------------------------------------------------------+
void InspectSELLPositions(double fTrend)
{
	bool bHaveSellOrders = SelectFirstSellPosition( _MagicNumber );

	if( !bHaveSellOrders ) return;
	
///	Print("InspectSELLPositions: first ticket is ", OrderTicket(), " / ", OrdersTotal());

	color colorCloseSell = Black; // Black means "don't close".
	if( IsTrendUp(fTrend))
	{
		Print("Closing All Sell position(s) - trend changed!");
		colorCloseSell = Red;
	}
	// Min(LOOKBACK-1, 1) wil guarantee proper operation in both real-Time
	// and closed-candle trading modes
	else if( IsCrossUp(0, MathMin(LOOKBACK-1, 1), arrRSI, arrRSILevelExitBuy) )
	{
		Print("Closing All Sell position(s) to cut losses");
		colorCloseSell = Orange;
	}

	// are we closing all SELL positions because things went bad?
	if( colorCloseSell != Black )
	{
		CloseAllPositions ( OP_SELL,
							_MagicNumber, 
							Ask,
							SLIPPAGE_ON_CLOSE, 
							colorCloseSell );
	}
	else // are we closing positions for profit?
	{
		bool bHaveTakeProfit = HaveTakeProfitSignal( OP_SELL );
		colorCloseSell = Green;

		if( bHaveTakeProfit )
		{
			while(bHaveSellOrders)
			{
				OrderSelect(OrdersTotal()-1, SELECT_BY_POS);
				int iTicket = OrderTicket();

				Print("Closing Sell position ", iTicket, " of ", OrdersTotal(), " for profit");
				double fLotsToCloseThisOrder = LotSizeNormalized(OrderLots()/2);

				if( !OrderClose(iTicket , OrderLots(), 
								Ask, SLIPPAGE_ON_CLOSE, colorCloseSell) )
				{
					Alert ( "InspectSELLPositions: Unable to Close Order #",
							iTicket, " Error # ", GetLastError() );
				}

				bHaveSellOrders = SelectNextSellPosition( _MagicNumber );
			}
		}
	}
}
//+------------------------------------------------------------------+
void TryOpenNewPositions(double fTrend)
{
	// remember whether we have already opened a trade this candle 
	// cause we never won't to open more than one per candle
	static bool s_bOpenedBUYOrderThisCandle = false;
	static bool s_bOpenedSELLOrderThisCandle = false;
	
	bool bTrade = false; // see below

	if(IsCandleStart(1))
	{
		s_bOpenedBUYOrderThisCandle = false;
		s_bOpenedSELLOrderThisCandle = false;

		bTrade = true;
	}
	else
	{
		bTrade = (g_iGoBack==0);
	}

	// if we opened a position this candle then take a rest until next candle
	if( s_bOpenedBUYOrderThisCandle || s_bOpenedSELLOrderThisCandle ) 
		return;

	// if we trade in real-time we look for open conditions constantly 
	// within the current candle. If we only trade on candle close (g_iGoBack=1)
	// then checking for open conditions constantly is not necessary 
	// because our conditions won't change until we get a new candle
	if( !bTrade )
		return;

	// on up-trend try to open a buy order
	if( IsTrendUp(fTrend) )
	{
		if( HaveBuySignal() )
		{
			OpenAtMarket (  OP_BUY, MaxLotSizeForMargin(UseableMargin),
							NormalizeDouble(Ask, 4), SLIPPAGE_ON_ENTER,
							_MagicNumber );
		}
	}
	// on down-trend try to open a sell order
	else if(IsTrendDown(fTrend))
	{
		if( HaveSellSignal() )
		{
			OpenAtMarket (  OP_SELL, MaxLotSizeForMargin(UseableMargin),
							NormalizeDouble(Bid, 4), SLIPPAGE_ON_ENTER,
							_MagicNumber );
		}
	}
}
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
	if( g_bDisabled ) return(0);

	CalcRSI();

	// check curr open positions for considtions like TP, SL, Trailing Stop, etc.
	double fTrend = CalcTrend();
	InspectBUYPositions(fTrend);
	InspectSELLPositions(fTrend);

	TryOpenNewPositions(fTrend);
	
	return(0);
}


//+------------------------------------------------------------------+
// an internal function - the main workhorse function behind these 
// exported function:
//	bool SelectFirstBuyPosition( int iMagicNumber )
//	bool SelectFirstSellPosition( int iMagicNumber )
//	bool SelectNextBuyPosition( int iMagicNumber )
//	bool SelectNextSellPosition( int iMagicNumber )
//
bool privSelectNextBuyOrSellPosition( int iMagicNumber, int iOrderType )
{
	int iFirstPosition;

	if( OP_BUY == iOrderType )
		iFirstPosition = s_iCurrBuyPosition;
	else if ( OP_SELL == iOrderType )
		iFirstPosition = s_iCurrSellPosition;
	else
	{
		Alert ( "SelectNextBuyOrSellPosition: Invalid OrderType ",
				iOrderType );

		return (false);
	}

	iFirstPosition = MathMin(iFirstPosition, OrdersTotal()-1);

	for( int iOrderPos = iFirstPosition; iOrderPos>=0; iOrderPos-- )
	{
		if( !OrderSelect( iOrderPos, SELECT_BY_POS ) )
		{
			Alert ( "IsBuyOrSellPosition: Unable to select order at position ",
					iOrderPos, " Error # ", GetLastError() );
			continue;
		}
		
		if( OrderType() == iOrderType &&
			OrderSymbol() == Symbol() &&
			OrderMagicNumber() == iMagicNumber )
		{
			if( OP_BUY == iOrderType )
			{
///				Print("privSelectNextBuyOrSellPosition: Found a BUY order at ", iOrderPos, ", Ticket is ", OrderTicket() );
				s_iCurrBuyPosition = iOrderPos-1;
			}
			else if ( OP_SELL == iOrderType )
			{
///				Print("privSelectNextBuyOrSellPosition: Found a SELL order at ", iOrderPos, ", Ticket is ", OrderTicket() );
				s_iCurrSellPosition = iOrderPos-1;
			}

			return (true);
		}
	}

	return (false);
}
//+------------------------------------------------------------------+
bool SelectFirstBuyPosition( int iMagicNumber )
{
	s_iCurrBuyPosition = OrdersTotal()-1;
	return(privSelectNextBuyOrSellPosition( iMagicNumber, OP_BUY ));
}
//+------------------------------------------------------------------+
bool SelectFirstSellPosition( int iMagicNumber )
{
	s_iCurrSellPosition = OrdersTotal()-1;
	return(privSelectNextBuyOrSellPosition( iMagicNumber, OP_SELL ));
}
//+------------------------------------------------------------------+
bool SelectNextBuyPosition( int iMagicNumber )
{
	return(privSelectNextBuyOrSellPosition( iMagicNumber, OP_BUY ));
}
//+------------------------------------------------------------------+
bool SelectNextSellPosition( int iMagicNumber )
{
	return(privSelectNextBuyOrSellPosition( iMagicNumber, OP_SELL ));
}
//+------------------------------------------------------------------+

