//+------------------------------------------------------------------+
//|                                            SeriesAnalysisLib.mqh |
//|                            Copyright  2007, Alexander Enchevich |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Alexander Enchevich"

#import "SeriesAnalysisLib.ex4"

//+------------------------------------------------------------------+
// will return true when called within iSecondsAllowed seconds of the 
// beginning of the current candle. In cases when the first tick of the 
// new candle arrives later than iSecondsAllowed true is still returned
// to indicate that the new candle has started.
bool IsCandleStart(int iSecondsAllowed);
//+------------------------------------------------------------------+
// Return Value: the total number of times that the samples in 
// arrFastSeries cross those in arrSlowSeries.
// Params:
//	iNumCrossesUp - upon return will contain number of times that Fast 
//					series aproaches Slow series from below and crosses it.
//	iNumCrossesDown - upon return will contain number of times that Fast 
//					series aproaches Slow series from above and crosses it.
//	iTickBackStart - analyze series starting from this tick (current tick is 0)
//	iTickBackEnd - analyze series up to this tick (current tick is 0)
//	arrFastSeries[] - samples for series 1
//	arrSlowSeries[] - samples for series 2
//
// NOTE: the function will not reset iNumCrossesUp and iNumCrossesDown. If you
// 		want them zeroed - do it urself b4 calling this function.
//
int CountNumCrosses(int& iNumCrossesUp,
					int& iNumCrossesDown,
					int iTickBackStart, 
					int iTickBackEnd, 
					double arrFastSeries[], 
					double arrSlowSeries[] );
//+------------------------------------------------------------------+
// was the last cross of the series a cross Down, i.e FastSeries
// approaches SlowSeries from above and crosses it downwards
bool IsCrossDown(int iTickBackStart,
				int iTickBackEnd, 
				double arrFastSeries[], 
				double arrSlowSeries[] );
//+------------------------------------------------------------------+
// was the last cross of the series a cross up, i.e FastSeries
// approaches SlowSeries from below and crosses it upwards
bool IsCrossUp(	int iTickBackStart,
				int iTickBackEnd, 
				double arrFastSeries[], 
				double arrSlowSeries[] );
//+------------------------------------------------------------------+
// Get the index of the first pair from the 2 series which lie past 
// the cross. 
// TODO !!!!!!!!!!! :
// test and verify whether this function will return 0 if the 
// first pair of samples in the 2 series have the same values.
int GetFirstCrossIndex(	int iTickBackStart,
						double arrFastSeries[], 
						double arrSlowSeries[] );
//+------------------------------------------------------------------+
int GetFirstCrossUpIndex(	int iTickBackStart,
							double arrFastSeries[], 
							double arrSlowSeries[] );
//+------------------------------------------------------------------+
int GetFirstCrossDownIndex(	int iTickBackStart,
							double arrFastSeries[], 
							double arrSlowSeries[] );
//+------------------------------------------------------------------+
// Function calculates a coefficient which is characteristic of the 
// convergence of the 2 series, based on the deviations between the 
// difference of each two adjacent samples of the series (which implies 
// that you need at least 2 samples in each series to calculate the 
// convergence, which is only natural of course - you can't calculate 
// the convergence of 2 points, you need at least 2 straight lines).
// 
// In a convergent or divergent pair of series these deviations will be
// gradually decreasing or increasing and their sum will be less than or 
// greater than the average deviation between 2 adjacent samples. In a 
// series which do not converge/diverge significantly this value will 
// close to the average deviation.
// Thus return values which are significantly > 1 or < -1 indicate 
// convergence / divergence (respectively) and values close to 0
// (e.g. between -1 and 1) indicate series running in parallel, or
// crossed in the middle.
// NOTE:This function is useful when determining whether two MAs are
// 		converging or diverging. 
double  SeriesConvergence (	int iTickBackStart,
							int iTickBackEnd, 
							double arrFastSeries[], 
							double arrSlowSeries[] );
//+------------------------------------------------------------------+

