//+------------------------------------------------------------------+
//|                                                   TradingLib.mq4 |
//|                            Copyright  2007, Alexander Enchevich |
//|                                                                  |
//| Note: Depends on CommonLib.                                      |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Alexander Enchevich"
#property link      ""

#import "TradingLib.ex4"

//+------------------------------------------------------------------+
// Returns true if we have no open BUY or SELL positions for our current 
// chart symbol and MagicNumber
bool IsEmptyPosition(int iMagicNumber);
//+------------------------------------------------------------------+
// Returns true and stuffs the corresponding ticket number into iTicket
// if we have at least one BUY position open for our current chart symbol 
// and with the specified MagicNumber.
bool HaveBuyPosition(int & iTicket, int iMagicNumber);
//+------------------------------------------------------------------+
// Returns true and stuffs the corresponding ticket number into iTicket
// if we have at least one SELL position open for our current chart symbol 
// and with the specified MagicNumber.
bool HaveSellPosition(int & iTicket, int iMagicNumber);
//+------------------------------------------------------------------+
// Four functions for looping thru currently open BUY and SELL positions.
// Example:
//
//	bool bContinue=SelectFirstBuyPosition( magiknum );
//	while(bContinue)
//	{
//		// ... process the selected order here ...
//
//		bContinue=SelectNextBuyPosition( magiknum );
//	}
//
bool SelectFirstBuyPosition( int iMagicNumber );
bool SelectFirstSellPosition( int iMagicNumber );
bool SelectNextBuyPosition( int iMagicNumber );
bool SelectNextSellPosition( int iMagicNumber );
//+------------------------------------------------------------------+
/*
	NOTE:	CloseAllPositions and OpenAtMarket are commented out in
			the header and in the library because there is a bug in
			MT4 (as of Build 202) whereby, when you back-test the 
			tester, you won't get on your charts any graphic indication
			(the blue/red arrows + trend line on the chart) of the 
			trade taking place if your OrderSend call was not executed 
			from within the main EA file.
			You'll have to manually copy and paste these functions from the
			library to the main file of your robot in order to get the arrows.

//+------------------------------------------------------------------+
// iOrderType is one of OP_... constants or -1 to close all of our
// positions (with matching MagicNumber).
void CloseAllPositions( int iOrderType, 
						int iMagicNumber, 
						double fPreferredClosePrice,
						int iSlippage, 
						color iColor );
//+------------------------------------------------------------------+
// iOrderType is one of OP_BUY or OP_SELL
// fLots will be normalized to 4, so for JPY normalize it to 2 yourself
// Color or arrow will be Blue for BUY and Red for Sell
// Returns:	the ticket or - on error (will also issue and Alert with 
//			the errno on error).
int OpenAtMarket (  int iOrderType,
					double fLots,
					double fPrice,
					int iSlippage, 
					int iMagicNumber );
*/
//+------------------------------------------------------------------+
// Given a max margin calculates the maximal lot size we can trade with.
// Will consider previous consecutive losses and will reduce lot size accordingly
double MaxLotSizeForMargin(double fUseableMargin);
//+------------------------------------------------------------------+
// Given a suggested user lot size will calculate a the nearest lot size 
// compliant to the server requirements for Min/Max lot size + increment step.
double LotSizeNormalized(double fSuggestedLotSize);
//+------------------------------------------------------------------+

