//+------------------------------------------------------------------+
//|                                                   TradingLib.mq4 |
//|                            Copyright  2007, Alexander Enchevich |
//|                                                                  |
//| Note: Depends on CommonLib.                                      |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Alexander Enchevich"
#property link      ""
#property library

#include <CommonLib.mqh>
#import "CommonLib.ex4"

//+------------------------------------------------------------------+
bool IsEmptyPosition(int iMagicNumber)
{
	for(int iOrderPos=0; iOrderPos<OrdersTotal(); iOrderPos++)
	{
		if( !OrderSelect( iOrderPos, SELECT_BY_POS ) )
		{
			Alert ( "IsEmptyPosition: Unable to select order at position ",
					iOrderPos, " Error # ", GetLastError() );
			continue;
		}

		if( OrderSymbol() == Symbol() && OrderMagicNumber() == iMagicNumber )
		{
			if( OrderType() == OP_BUY || OrderType() == OP_SELL )
				return(false);
	
			Print("IsEmptyPosition: Ticket ", OrderTicket(), " considered EMPTY POSITION");
		}
		else
			Print( "IsEmptyPosition: Ticket ", OrderTicket(), " at position ",
					iOrderPos, " is not ", Symbol(), " mgk:", iMagicNumber,
					" but ", OrderSymbol(), " mgk:", OrderMagicNumber(),
					" Type ", OrderType() );
	}

	return(true);
}
//+------------------------------------------------------------------+
// an internal function - the main workhorse function behind these 
// exported function:
//	bool HaveBuyPosition(int & iTicket, int iMagicNumber)
//	bool HaveSellPosition(int & iTicket, int iMagicNumber)
// 
bool privIsBuyOrSellPosition(int & iTicket, int iMagicNumber, int iOrderType )
{
	if( iOrderType != OP_BUY && iOrderType != OP_SELL )
		return(false);

	for( int iOrderPos=OrdersTotal(); iOrderPos>0; iOrderPos-- )
	{
		if( !OrderSelect( iOrderPos-1, SELECT_BY_POS ) )
		{
			Alert ( "IsBuyOrSellPosition: Unable to select order at position ",
					iOrderPos-1, " Error # ", GetLastError() );
			continue;
		}
		
		if( OrderType() == iOrderType &&
			OrderSymbol() == Symbol() &&
			OrderMagicNumber() == iMagicNumber )
		{
			iTicket = OrderTicket();
			
			return (true);
		}
	}

	return (false);
}
//+------------------------------------------------------------------+
bool HaveBuyPosition(int & iTicket, int iMagicNumber)
{
	return( privIsBuyOrSellPosition( iTicket, iMagicNumber, OP_BUY ) );
}
//+------------------------------------------------------------------+
bool HaveSellPosition(int & iTicket, int iMagicNumber)
{
	return( privIsBuyOrSellPosition( iTicket, iMagicNumber, OP_SELL ) );
}
/*
int OpenAtMarket (  int iOrderType,
					double fLots,
					double fPrice,
					int iSlippage, 
					int iMagicNumber )
{
	if(iOrderType != OP_BUY && iOrderType != OP_SELL)
		return(-1);

// TODO : check for other limitations of OrderSend
// TODO : Implement context locking!
	if(fLots < 0.1)
		return(-1);

	string strOperation = OperationToStr( iOrderType );

	double fStopLoss = Ask-10000*Point;
	double fTakeProfit = Ask+999*Point;
	color iColor = RoyalBlue;
	
	if( OP_SELL == iOrderType )
	{		
		fStopLoss = Bid+10000*Point;
		fTakeProfit = Bid-999*Point;
		iColor = OrangeRed;
	}

	int iTicket = 
	OrderSend(	Symbol(), iOrderType, 
				NormalizeDouble(fLots, 1), NormalizeDouble(fPrice, 4),
				iSlippage, fStopLoss, fTakeProfit,
				strOperation, iMagicNumber, 0, iColor );

	if( -1 == iTicket )
	{
		Alert("OpenAtMarket: OrderSend Error ", GetLastError() );
	}
	
	return(iTicket);
}
//+------------------------------------------------------------------+
void CloseAllPositions( int iOrderType, 
						int iMagicNumber,
						double fPreferredClosePrice,
						int iSlippage, 
						color iColor )
{
	for( int iOrderPos=OrdersTotal(); iOrderPos>0; iOrderPos-- )
	{
		if( !OrderSelect( iOrderPos-1, SELECT_BY_POS ) )
		{
			Alert ( "CloseAllPositions: Unable to select order at position ", 
					iOrderPos-1, " Error # ", GetLastError() );
		}
		else if(OrderSymbol() == Symbol() &&
				OrderMagicNumber() == iMagicNumber )
		{
			bool bCloseAll = (iOrderType==-1);

			if( bCloseAll || OrderType() == iOrderType )
			{
				int iTicket = OrderTicket();
				if( !OrderClose(iTicket, OrderLots(), fPreferredClosePrice, iSlippage, iColor) )
				{
					Alert ( "CloseAllPositions: Unable to Close Order #", iTicket,
							" at position ", iOrderPos-1,
							" Error # ", GetLastError() );
				}
			}
		}
	}
}
*/
//+------------------------------------------------------------------+
double LotSizeNormalized(double fSuggestedLotSize)
{
	double fMinLot = MarketInfo(Symbol(), MODE_MINLOT);
	double fMaxLot = MarketInfo(Symbol(), MODE_MAXLOT);
	double fLotStep = MarketInfo(Symbol(), MODE_LOTSTEP);

	double fOrigLotSize = fSuggestedLotSize;

	// ensure fSuggestedLotSize is between MinLot and MaxLot
	fSuggestedLotSize = 
		MathMin(MathMax(fMinLot, fSuggestedLotSize), fMaxLot);

	if(fSuggestedLotSize < 0.000001)
		return (0.0);

	// ensure our lot is an integer multiple of the allowed LotStep	
	int iNumStepsInDesiredLotSize = 
		NormalizeDouble((fSuggestedLotSize - fMinLot)/fLotStep, 0);

	return (iNumStepsInDesiredLotSize*fLotStep + fMinLot);
}
//+------------------------------------------------------------------+
// TODO: See if I need to improve code in accordance with what was
// suggested on the mql forum regarding orderig of completed orders:
//		http://www.metaquotes.ru/forum/8037/
// Apparently it is possible that looping backwards (i--) is not good 
// enough to guarantee that we're going backwards (chronologically) thru 
// the list of completed orders
//
double MaxLotSizeForMargin(double fUseableMargin)
{
	//---- select lot size
	double fLotSize=NormalizeDouble(AccountFreeMargin()*fUseableMargin/1000.0,1);
	//---- calculate number of continuous losing orders
	int    iLosses=0;				// number of continuous losing orders
	int    iNumOrders = HistoryTotal();	// history orders total

	for(int i=iNumOrders-1; i>=0; i--)
	{
		if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false)
		{
			Alert("MaxLotSizeForMargin: Error in history!");
			// reduce lot size on History error, just in case...
			if(iLosses < 3) 
				iLosses = 3;
			break; 
		}

		if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
		//----
		if(OrderProfit()>0) break;
		if(OrderProfit()<0) iLosses++;
	}

	// If we have more than 2 consecutive losses in a row - reduce size of lot
	if(iLosses>2) 
		fLotSize=LotSizeNormalized(fLotSize/(iLosses-1));

	return(fLotSize);
}
//+------------------------------------------------------------------+

